import { Typography, useEds, Table, Pagination } from '@equinor/eds-core-react';
import { getCoreRowModel, getSortedRowModel, getFacetedRowModel, getFacetedUniqueValues, getFacetedMinMaxValues, getFilteredRowModel, getPaginationRowModel, useReactTable } from '@tanstack/react-table';
import { useVirtualizer } from '@tanstack/react-virtual';
import { useState, useEffect, useMemo, useRef, useCallback } from 'react';
import { TableHeaderRow } from './components/TableHeaderRow.js';
import { TableRow } from './components/TableRow.js';
import { TableProvider } from './EdsDataGridContext.js';
import { jsx, jsxs, Fragment } from 'react/jsx-runtime';

/* eslint-disable @typescript-eslint/no-non-null-assertion */
function EdsDataGrid({
  rows,
  columns,
  columnResizeMode,
  pageSize,
  rowSelection,
  selectedRows,
  enableColumnFiltering,
  debug,
  enablePagination,
  enableSorting,
  stickyHeader,
  onSelectRow,
  caption,
  enableVirtual,
  virtualHeight,
  columnVisibility,
  columnVisibilityChange,
  emptyMessage,
  columnOrder,
  cellClass,
  cellStyle,
  rowClass,
  rowStyle,
  headerClass,
  headerStyle,
  externalPaginator,
  onSortingChange,
  manualSorting,
  sortingState,
  columnPinState,
  scrollbarHorizontal,
  width,
  minWidth,
  height,
  getRowId,
  rowVirtualizerInstanceRef
}) {
  const [sorting, setSorting] = useState(sortingState ?? []);
  const [selection, setSelection] = useState(selectedRows ?? {});
  const [columnPin, setColumnPin] = useState(columnPinState ?? {});
  const [columnFilters, setColumnFilters] = useState([]);
  const [visible, setVisible] = useState(columnVisibility ?? {});
  const [globalFilter, setGlobalFilter] = useState('');
  const [columnOrderState, setColumnOrderState] = useState([]);
  const [page, setPage] = useState({
    pageIndex: 0,
    pageSize: pageSize ?? 25
  });
  useEffect(() => {
    setVisible(columnVisibility ?? {});
  }, [columnVisibility, setVisible]);
  useEffect(() => {
    setColumnPin(s => columnPinState ?? s);
  }, [columnPinState]);
  useEffect(() => {
    setSorting(sortingState);
  }, [sortingState]);
  useEffect(() => {
    setSelection(selectedRows ?? {});
  }, [selectedRows]);

  /**
   * By default, the filter-function accepts single-value filters. This adds multi-filter functionality out of the box.
   */
  const _columns = useMemo(() => {
    return columns.map(column => {
      if (column.filterFn || column.enableColumnFilter === false) {
        return column;
      }
      /* istanbul ignore next */
      return {
        ...column,
        filterFn: (row, columnId, filterValue) => {
          if (debug) {
            console.log('filterFn', row, columnId, filterValue);
          }
          if (!filterValue || (Array.isArray(filterValue) || typeof filterValue === 'string') && filterValue.length === 0) {
            return true;
          }
          const value = row.getValue(columnId) ?? 'NULL_OR_UNDEFINED';
          if (Array.isArray(filterValue)) {
            const numeric = filterValue.some(v => typeof v === 'number');
            if (numeric) {
              const [start, end] = filterValue;
              return Number(value) >= (isNaN(start) ? 0 : start) && Number(value) <= (!end || isNaN(end) ? Infinity : end);
            } else {
              const validFilterValue = filterValue.filter(v => !!v);
              if (validFilterValue.length === 0) return true;
              return filterValue.includes(value);
            }
          }
          return value === filterValue;
        }
      };
    });
  }, [debug, columns]);

  /**
   * Set up default table options
   */
  const options = {
    data: rows,
    columns: _columns,
    defaultColumn: {
      cell: context => {
        return /*#__PURE__*/jsx(Typography, {
          style: {
            overflow: 'hidden',
            whiteSpace: 'nowrap',
            textOverflow: 'ellipsis'
          },
          group: "table",
          variant: "cell_text",
          children: String(context.getValue() ?? '')
        });
      }
    },
    columnResizeMode: columnResizeMode,
    state: {
      sorting,
      columnPinning: columnPin,
      rowSelection: selection,
      columnOrder: columnOrderState
    },
    onSortingChange: changes => {
      if (onSortingChange) {
        onSortingChange(changes);
      }
      setSorting(changes);
    },
    enableColumnFilters: !!enableColumnFiltering,
    enableFilters: !!enableColumnFiltering,
    enableSorting: enableSorting ?? false,
    manualSorting: manualSorting ?? false,
    enableColumnResizing: !!columnResizeMode,
    getCoreRowModel: getCoreRowModel(),
    getSortedRowModel: getSortedRowModel(),
    debugTable: debug,
    debugHeaders: debug,
    debugColumns: debug,
    enableRowSelection: rowSelection ?? false,
    enableColumnPinning: true,
    enablePinning: true,
    getRowId
  };
  useEffect(() => {
    if (columnOrder && columnOrder.length > 0) {
      setColumnOrderState(columnOrder ?? []);
    }
  }, [columnOrder]);

  /**
   * Set up handlers for rowSelection
   */
  if (rowSelection ?? false) {
    options.onRowSelectionChange = updaterOrValue => {
      if (onSelectRow) {
        onSelectRow(updaterOrValue);
      }
      setSelection(updaterOrValue);
    };
  }

  /**
   * Set up config for column filtering
   */
  if (enableColumnFiltering) {
    options.state.columnFilters = columnFilters;
    options.state.globalFilter = globalFilter;
    options.onColumnFiltersChange = setColumnFilters;
    options.onGlobalFilterChange = setGlobalFilter;
    options.getFacetedRowModel = getFacetedRowModel();
    options.getFacetedUniqueValues = getFacetedUniqueValues();
    options.getFacetedMinMaxValues = getFacetedMinMaxValues();
    options.getFilteredRowModel = getFilteredRowModel();
  }

  /**
   * Set up config for pagination
   */
  if (enablePagination ?? false) {
    options.state.pagination = page;
    options.getPaginationRowModel = getPaginationRowModel();
  }

  /**
   * Set up config to handle column visibility controls
   */
  if (columnVisibility) {
    options.state.columnVisibility = visible;
    options.onColumnVisibilityChange = vis => {
      let updated;
      if (typeof vis === 'function') {
        updated = vis(visible);
      } else {
        updated = vis;
      }
      if (columnVisibilityChange) columnVisibilityChange(updated);
      setVisible(updated);
    };
  }
  useEffect(() => {
    setPage(prev => ({
      ...prev,
      pageSize: pageSize ?? 25
    }));
  }, [pageSize]);
  const table = useReactTable(options);
  let parentRefStyle = {};

  /**
   * Style the parent container to enable virtualization.
   * By not setting this, the virtual-scroll will always render every row, reducing computational overhead if turned off.
   */
  if (enableVirtual) {
    parentRefStyle = {
      height: height ?? virtualHeight ?? 500,
      overflow: 'auto',
      position: 'relative'
    };
  }
  const parentRef = useRef(null);

  /**
   * Virtualization setup
   */
  const {
    density
  } = useEds();
  const estimateSize = useCallback(() => {
    return density === 'compact' ? 32 : 48;
  }, [density]);
  const virtualizer = useVirtualizer({
    count: table.getRowModel().rows.length,
    getScrollElement: () => parentRef.current,
    estimateSize
  });
  if (rowVirtualizerInstanceRef) rowVirtualizerInstanceRef.current = virtualizer;
  const virtualRows = virtualizer.getVirtualItems();
  const paddingTop = virtualRows.length ? virtualRows[0].start : 0;
  const paddingBottom = virtualRows.length ? virtualizer.getTotalSize() - virtualRows[virtualRows.length - 1].end : 0;

  // These classes are primarily used to allow for feature-detection in the test-suite
  const classList = {
    'sticky-header': !!stickyHeader,
    virtual: !!enableVirtual,
    paging: !!enablePagination
  };
  return /*#__PURE__*/jsxs(TableProvider, {
    cellClass: cellClass,
    cellStyle: cellStyle,
    rowClass: rowClass,
    rowStyle: rowStyle,
    headerClass: headerClass,
    headerStyle: headerStyle,
    table: table,
    enableSorting: !!enableSorting,
    enableColumnFiltering: !!enableColumnFiltering,
    stickyHeader: !!stickyHeader,
    children: [/*#__PURE__*/jsxs("div", {
      className: "table-wrapper",
      style: {
        height: height ?? 'auto',
        ...parentRefStyle,
        width: scrollbarHorizontal ? width ?? '100%' : 'auto',
        overflow: 'auto',
        contain: 'layout paint style'
      },
      ref: parentRef,
      children: [/*#__PURE__*/jsxs(Table, {
        className: Object.entries(classList).filter(([, k]) => k).map(([k]) => k).join(' '),
        style: {
          tableLayout: scrollbarHorizontal ? 'fixed' : 'auto',
          width: table.getTotalSize(),
          minWidth: scrollbarHorizontal ? minWidth : 'auto'
        },
        children: [caption && /*#__PURE__*/jsx(Table.Caption, {
          children: caption
        }), /*#__PURE__*/jsx(Table.Head, {
          sticky: stickyHeader,
          children: table.getHeaderGroups().map(headerGroup => /*#__PURE__*/jsx(TableHeaderRow, {
            table: table,
            headerGroup: headerGroup,
            columnResizeMode: columnResizeMode,
            deltaOffset: table.getState().columnSizingInfo.deltaOffset
          }, headerGroup.id))
        }), /*#__PURE__*/jsxs(Table.Body, {
          style: {
            backgroundColor: 'inherit'
          },
          children: [table.getRowModel().rows.length === 0 && emptyMessage && /*#__PURE__*/jsx(Table.Row, {
            children: /*#__PURE__*/jsx(Table.Cell, {
              colSpan: table.getFlatHeaders().length,
              children: emptyMessage
            })
          }), enableVirtual && /*#__PURE__*/jsxs(Fragment, {
            children: [paddingTop > 0 && /*#__PURE__*/jsx(Table.Row, {
              "data-testid": "virtual-padding-top",
              className: 'virtual-padding-top',
              style: {
                pointerEvents: 'none'
              },
              children: /*#__PURE__*/jsx(Table.Cell, {
                style: {
                  height: `${paddingTop}px`
                }
              })
            }), virtualRows.map(row => /*#__PURE__*/jsx(TableRow, {
              row: table.getRowModel().rows[row.index]
            }, row.index)), paddingBottom > 0 && /*#__PURE__*/jsx(Table.Row, {
              "data-testid": "virtual-padding-bottom",
              className: 'virtual-padding-bottom',
              style: {
                pointerEvents: 'none'
              },
              children: /*#__PURE__*/jsx(Table.Cell, {
                style: {
                  height: `${paddingBottom}px`
                }
              })
            })]
          }), !enableVirtual && table.getRowModel().rows.map(row => /*#__PURE__*/jsx(TableRow, {
            row: row
          }, row.id))]
        })]
      }), externalPaginator ? externalPaginator : enablePagination && /*#__PURE__*/jsx("div", {
        style: {
          maxWidth: `${table.getTotalSize()}px`
        },
        children: /*#__PURE__*/jsx(Pagination, {
          totalItems: table.getFilteredRowModel().rows.length,
          withItemIndicator: true,
          itemsPerPage: page.pageSize,
          onChange: (e, p) => setPage(s => ({
            ...s,
            pageIndex: p - 1
          })),
          defaultPage: 1
        })
      })]
    }), debug && enableVirtual && /*#__PURE__*/jsxs("span", {
      children: ["Visible items: ", virtualizer.range.startIndex, " -", ' ', virtualizer.range.endIndex, " / ", rows.length]
    })]
  });
}

export { EdsDataGrid };
